function [Ctrl,Prob] = find_controls_probabilities(T,D_A,D_B,D_b,U)
%find feasible control laws and corresponding probabilities:
%Ctrl{si,sj,sk} is the control for going from cell i to j, given that i was entered from k
%Prob(si,sj,sk) is the corresponding probability (when applying above control, with what probability is cell j entered?)

prec=1e5*eps;   %for expressing strict inequalities in linprog

time_step=1e-3; %for backwards integration

%find U_A, U_b for set U containing control bounds
H=cddmex('hull',struct('V',U)); %H-repres. of polytope U
U_A=H.A;
U_b=-H.B;
m=size(U_A,2);  %number of control inputs

% for si=[7]
for si=1:length(T.Q)
    si
    V=T.Vert{si};   %matrix with vertices (on rows) of current state (for easier acces)
    M_inv_V=inv([V';ones(1,3)]);  %inverse of matrix of vertices, bounded with ones, useful for creating controllers
    centr=mean(V);  %centroid
    F_n=zeros(3,2); %matrix with outer normals (on rows) for facets of current triangle
    Neigh=zeros(1,3); %vector with 3 elements, position i gives the neighbor index corresponding to exit facet i (neighbor is 0 for boundaries (unfeasible for exit) facets)
    for sj=setdiff( find(T.adj(si,:)),si);    %indices of neighbors (excluding cell i)
        [ignore,ex_f]=setdiff(V,T.Vert{sj},'rows');  %a single vertex of si is not in sj; it's index is also index of exit facet
        Neigh(ex_f)=sj; %vector will be used for finding feasible exit facets in nondeterministic transitions
    end
    for j=1:3   %for each vertex, find outernormal of opposite facet
        v_ind=setdiff(1:3,j); %the other 2 vertices
        v_f=V(v_ind,:);
        n_j=[v_f(2,2)-v_f(1,2), v_f(1,1)-v_f(2,1)];  %normal (row vector)
        vect=v_f(1,:)-V(j,:); %vector from vertex j to first vertex of facet j
        F_n(j,:) = sign(n_j * vect') * n_j; %outer normal (for unity outer normal, use "F_n(j,:) = sign(n_j * vect') * n_j/norm(n_j)" - more representation errors)
    end

    for sj=setdiff( find(T.adj(si,:)),si);    %indices of neighbors (excluding cell i)
        %test if possible to go from i to just j (exit through a single facet)
        det_si_sj=1;    %flag for assuming it's possible to deterministically go from si to sj
        %find common facet of s_i and s_j (which is the exit facet for s_i)
        [ignore,ex_f]=setdiff(V,T.Vert{sj},'rows');  %a single vertex of si is not in sj; it's index is also index of exit facet
        C_c=zeros(m,3); %for storing controls
        for k=1:3    %for all vertices of s_i check feasibility by checking non-emptiness of optimisation set (imposed by restrictions)
            in_f=setdiff(1:3,[k,ex_f]);  %indices of facets, different than the exit facet, containing vertex k (stay inside facets)
            A_check=[U_A ; -F_n(ex_f,:)*D_B ; F_n(in_f,:)*D_B]; %matrices A_check, B_check contain restrictions for control u: A_check u <= B_check
            B_check=[-U_b ; F_n(ex_f,:)*(D_A*V(k,:)'+D_b)-prec ; -F_n(in_f,:)*(D_A*V(k,:)'+D_b)];   %prec is for strict inequality (strict exit through facet)
            [uval,fval,exitflag]=linprog(-F_n(ex_f,:)*D_B, A_check , B_check ,[],[],[],[],[],optimset('Display','off'));    %try minimization (maximization of speed projection on normal of exit facet)
            if exitflag~=1  %optimization didn't converged to a feasible solution, so it's not possible to find a control
                det_si_sj=0;
                break   %exit only to neighbor j not feasible
            else
                C_c(:,k)=uval;   %current control
            end
        end

        if det_si_sj==1      %we have a deterministic transiton (prob. 1), no matter of previous state sk
            ctrl_temp=C_c*M_inv_V;  %controls [F,g]
            for sk=setdiff( find(T.adj(:,si)),si)';  %possible previous states sk (should always sk~=si ?)
                Ctrl{si,sj,sk}=ctrl_temp;   %same control, no matter sk
                Prob(si,sj,sk)=1;   %probability is 1
            end
            fprintf('\nDeteministic transition from cell %d to %d possible.\n',si,sj);
            continue    %continue with next neighbor sj
        end
        
        %else (det_si_sj==0), find a control driving to a set of facets that includes ex_f, and after that compute probability of reaching sj (given
        %that sk was the previous state); sk is not used until we have to compute probability
        fprintf('\nDeteministic transition from cell %d to %d IMPOSSIBLE; trying nondeterministic.\n',si,sj);
        for sk=setdiff( find(T.adj(:,si)),si)';  %possible previous states sk
            if sk==sj   %try only vertices of common facet (ex_f)
                fprintf('Probabilistic transition from cell %d to %d, sk=%d.\n',si,sj,sk);
                bad_vertex=[];   %showing vertices where the control set is unfeasible
                C_c=zeros(m,3); %for storing controls
                for k=setdiff(1:3,ex_f)    %for all vertices of the exit facet (2 vertices)
                    in_f=setdiff(1:3,[k,ex_f]);  %index of facet, different than the exit facet, containing vertex k (stay inside facet)
                    A_check=[U_A ; -F_n(ex_f,:)*D_B ; F_n(in_f,:)*D_B]; %matrices A_check, B_check contain restrictions for control u: A_check u <= B_check
                    B_check=[-U_b ; F_n(ex_f,:)*(D_A*V(k,:)'+D_b)-prec ; -F_n(in_f,:)*(D_A*V(k,:)'+D_b)];   %prec is for strict inequality (strict exit through facet)
                    [uval,fval,exitflag]=linprog(-F_n(ex_f,:)*D_B, A_check , B_check ,[],[],[],[],[],optimset('Display','off'));    %try minimization (maximization of speed projection on normal of exit facet)
                    if exitflag~=1  %optimization didn't converged to a feasible solution, so it's not possible to find a control
                        bad_vertex=[bad_vertex k];    %add k (current vertex)
                        A_check=[U_A ; F_n(in_f,:)*D_B]; %only restrictions inside of si
                        B_check=[-U_b ; -F_n(in_f,:)*(D_A*V(k,:)'+D_b)];
                        [uval,fval,exitflag]=linprog(-F_n(ex_f,:)*D_B, A_check , B_check ,[],[],[],[],[],optimset('Display','off'));
                        if exitflag~=1  %optimization didn't converge
                            [uval,fval,exitflag]=linprog(-F_n(ex_f,:)*D_B, U_A , -U_b ,[],[],[],[],[],optimset('Display','off'));   %no other restrictions than U
                            if exitflag~=1  %just for test - this shoudn't happen, since U is nonempty
                                fprintf('Error - is set U empty?');
                            end
                        end
                    end
                    C_c(:,k)=uval;   %current control
                end
                ctrl_temp=C_c*M_inv_V;  %controls [F,g]
%                 bad_vertex
%                 length(bad_vertex)
                switch length(bad_vertex)
                    case 0  %probability of exit 1 (both vertices have feasible controls)
                        Ctrl{si,sj,sk}=ctrl_temp;   %control when previous state sk
                        Prob(si,sj,sk)=1;   %probability is 1
                    case 1  %only one vertex feasible, find probability
                        Ctrl{si,sj,sk}=ctrl_temp;   %control when previous state sk
%                         [si sj sk]
%                         [h_exit_vvf] = vect_field_vertex(V, ctrl_temp,D_A,D_B,D_b);
                        prob_reentry=find_exit_probability_reentry(T,V,F_n,sj,D_A,D_B,D_b,ctrl_temp,time_step);   %***compute probability
                        Prob(si,sj,sk)= prob_reentry;
                    case 2  %probability zero, cannot enter sj
                        Ctrl{si,sj,sk}=ctrl_temp;   %control when previous state sk
                        Prob(si,sj,sk)=0;   %probability is 0
                    otherwise
                        fprintf('Error - impossible case');
                end
                fprintf('\tProbability: %g\n',Prob(si,sj,sk));

            else %(sk~=sj)
                fprintf('Probabilistic transition from cell %d to %d, when previous state was %d.\n',si,sj,sk);
                %construct exit_set: at most 3 sets: ex_f + one facet (left, respectively right), and all three facets (less than 3 if cell is near boundary of space)
%                 [ignore,entry_f]=setdiff(V,T.Vert{sk},'rows');  %a single vertex of si is not in sj; it's index is also index of exit facet
                ot_f=setdiff(find(Neigh),ex_f);    %other feasible facets, if any (feasible means we cannot leave state space boundaries through them,or not hit obstacles)
                switch length(ot_f) %maximum 2 other facets
                    case 0
                        exit_set={ex_f};    %this was already tested and it wasn't feasible
                        Ctrl{si,sj,sk}=zeros(m,3)*M_inv_V;
                        Prob(si,sj,sk)=0;   %probability is 0
                        continue    %continue with next sk
                    case 1
                        exit_set={[ex_f, ot_f]};   %only one combination to test
                    case 2  %all three facets have neighbors
                        exit_set={[ex_f, ot_f(1)], [ex_f, ot_f(2)], [1:3]};
                end
                C_c_cell=cell(1,length(exit_set)); %for storing controls
                prob_temp=zeros(1,length(exit_set)); %for probabilities
                bad_set=[]; %unfeasible exit set
                for ex_ind=1:length(exit_set)   %for each exit facets set
                    C_c_cell{ex_ind}=zeros(m,3);
                    for k=1:3    %for each vertex, find feasible controls
                        %first try ALL restrictions
                        in_f=setdiff(1:3,[k,ex_f]);  %index of facet, different than the exit facet, containing vertex k (stay inside facet)
                        A_check=[U_A ; -F_n(ex_f,:)*D_B ; F_n(in_f,:)*D_B]; %matrices A_check, B_check contain restrictions for control u: A_check u <= B_check
                        B_check=[-U_b ; F_n(ex_f,:)*(D_A*V(k,:)'+D_b)-prec ; -F_n(in_f,:)*(D_A*V(k,:)'+D_b)];   %prec is for strict inequality (strict exit through facet)
                        [uval,fval,exitflag]=linprog(-F_n(ex_f,:)*D_B, A_check , B_check ,[],[],[],[],[],optimset('Display','off'));    %try minimization (maximization of speed projection on normal of exit facet)
                        if exitflag~=1  %optimization didn't converged to a feasible solution, so it's not possible to find a control
                            %now try LESS restrictions (without exit facet)
                            A_check=[U_A ; F_n(in_f,:)*D_B]; %only restrictions inside of si
                            B_check=[-U_b ; -F_n(in_f,:)*(D_A*V(k,:)'+D_b)];
                            [uval,fval,exitflag]=linprog(-F_n(ex_f,:)*D_B, A_check , B_check ,[],[],[],[],[],optimset('Display','off'));
                            if exitflag~=1  %optimization didn't converge
                                %try even LESS restrictions (fewer inside facets - corresponding to current exit_set, plus exit facet)
                                in_f=setdiff(1:3, union(k,exit_set{ex_ind}));  %stay inside facets (if any)
                                A_check=[U_A ; -F_n(ex_f,:)*D_B ; F_n(in_f,:)*D_B]; %matrices A_check, B_check contain restrictions for control u: A_check u <= B_check
                                B_check=[-U_b ; F_n(ex_f,:)*(D_A*V(k,:)'+D_b)-prec ; -F_n(in_f,:)*(D_A*V(k,:)'+D_b)];   %prec is for strict inequality (strict exit through facet)
                                [uval,fval,exitflag]=linprog(-F_n(ex_f,:)*D_B, A_check , B_check ,[],[],[],[],[],optimset('Display','off'));    %try minimization (maximization of speed projection on normal of exit facet)
                                if exitflag~=1  %optimization didn't converged to a feasible solution, so it's not possible to find a control
                                    %try even LESS restrictions (few inside facets - corresponding to current exit_set, without exit facet)
                                    %in_f=setdiff(1:3, union(k,exit_set{ex_ind}));  %stay inside facets (if any) - set in_f already computed in previous "if"
                                    A_check=[U_A ; F_n(in_f,:)*D_B]; %only stay inside restrictions, we'll check is an equilibrium is not inside after this
                                    B_check=[-U_b ; -F_n(in_f,:)*(D_A*V(k,:)'+D_b)];
                                    [uval,fval,exitflag]=linprog(-F_n(ex_f,:)*D_B, A_check , B_check ,[],[],[],[],[],optimset('Display','off'));
                                    if exitflag~=1  %optimization didn't converge
                                        bad_set=[bad_set,ex_ind];
                                        break   %continue with next exit_set, this one is not feasible
                                    end
                                end
                            end
                        end
                        C_c_cell{ex_ind}(:,k)=uval;   %current control
                    end
                    
                    if ~ismember(ex_ind,bad_set) %bad_set does not contain current ex_ind, so we might have feasible controls
                        %check if with current control an equilibrium inside triangle exists - if no, go ahead with probability, otherwise add ex_ind to bad_set
                        r_v=D_A*V'+D_B*C_c_cell{ex_ind}+repmat(D_b,1,3); %resulting velocities at vertices of current triangle (on columns)
                        ind_ch=convhull(r_v(1,:),r_v(2,:));
                        r_v=r_v(:,ind_ch(1:end-1));
                        if inpolygon(0,0,r_v(1,:),r_v(2,:))  %equilibrium point inside or on boundary, not feasible
                            bad_set=[bad_set,ex_ind];  %probability remains zero (in prob_temp)
                        else
                            ctrl_temp=C_c_cell{ex_ind}*M_inv_V;  %controls [F,g]
                            %disp('feasible, have to integrate')
                            %***backwards integration to find probability (prob_temp(ex_ind))
%                             [si sj sk]
%                             [h_exit_vvf] = vect_field_vertex(V, ctrl_temp,D_A,D_B,D_b);
                            prob_temp(ex_ind) = find_exit_probability(T,V,F_n,sj,sk,D_A,D_B,D_b,ctrl_temp,time_step);
%                             [prob_temp(ex_ind),points_in,points_out] = find_exit_prob_points(V,F_n,entry_f,ex_f,D_A,D_B,D_b,ctrl_temp,time_step);
%                             pause
                        end
                    end     %else, probability remains zero
                end
                [Prob(si,sj,sk), ex_ind]=max(prob_temp);    %exit set giving the maximum probability (possible 0 probability)
                %(we don't need probabilities for reaching other states than sj, because they will be computed (and maximized) specifically for those states)
                Ctrl{si,sj,sk}=C_c_cell{ex_ind}*M_inv_V;    %controls
                fprintf('\tProbability: %g\n',Prob(si,sj,sk));
            end
        end
    end
end

return



%don't need from here, just pieces
V=T.Vert{1};
[h_exit_vvf] = vect_field_vertex(V, Ctrl{1,4,2},D_A,D_B,D_b);

V=T.Vert{2};
[h_exit_vvf] = vect_field_vertex(V, Ctrl{2,1,1},D_A,D_B,D_b);
[h_exit_vvf] = vect_field_vertex(V, Ctrl{2,1,10},D_A,D_B,D_b);

%compute vector field
[Xg,Yg,Gx,Gy] = vf_polytope2(V, controls, 10, 10); %values of vector field, choosing a grid with 10 points on x- and y-axis
%v.f. only at veritces
[Xv,Yv,Gvx,Gvy] = vf_vertex_polytope2(V, controls);

%plot vector fields
for c=1:length(controls)-2
    h_exit_vf=quiver(Xg,Yg,Gx{c},Gy{c},colors(c),'Linewidth',1);
    set(h_exit_vf,'AutoScaleFactor',0.3);    %auto scale with smaller factor (default is 0.9)
    h_exit_vvf=quiver(Xv,Yv,Gvx{c},Gvy{c},colors(c),'Linewidth',2);
    set(h_exit_vvf,'AutoScaleFactor',0.2);    %auto scale with smaller factor (default is 0.9)
end

% %represent drift
% [Xg,Yg]=meshgrid(linspace(min(X(:,1)),max(X(:,1)),15) , linspace(min(X(:,2)),max(X(:,2)),15));  %grid to represent v.f. (15 points on each dir)
% Gx = D_A(1,1)*Xg + D_A(1,2)*Yg + D_b(1,1);    %values of projection of v.f. on the first axis (X,Y are matrices, that's why we write result as sum of products with scalars)
% Gy = D_A(2,1)*Xg + D_A(2,2)*Yg + D_b(2,1);    %projection of v.f. on the second axis
% h_vf=quiver(Xg,Yg,Gx,Gy,'k');  %vector field and a handle to it
